GroupObjectClass

Enum GroupObjectClass 

pub enum GroupObjectClass {
    Group,
    GroupOfNames,
    GroupOfUniqueNames,
}
Possible choices for the objectClass attribute of group entries.

GroupOfNames is currently regarded as the default variant and is thus the one being returned by the impl of Default.

Variants



Group



GroupOfNames



GroupOfUniqueNames